##----------------------------------------------------------------------------##
##=Purpose: Exercise 4.1 of Beyond Classic Designs & Analysis for Health      ##
##          Data in Singapore (27 June to 8 July 2022)                        ##
##
##=Data required: 
##  - chap8_CASS.dta 
##
##=R packages/libraries required:
##  - design2phase: 
##    * PowerIIphase: To generate power
##    * PlotPower: To plot power curve
##  - haven:
##    * read_dta: To read in recent versions of STATA dta files
##  - knitr:
##    * kable: Generate nice tables
##  - survey:
##    * twophase: Specify the two stage design information
##    * svyglm: Perform glm analysis with sampling weights
##
##=Created by:
##  - Francesca Graziano 
##  - Zilu Feng
##  - Zihan 
##  - Chuen Seng Tan
##  - Marie Reilly
##
##=Changes (if any):
##  -DDMMYYYY: Describe
##
##----------------------------------------------------------------------------##

##-Set the working directory to the data folder where the datasets are saved:
setwd("D:/nbox/Education/Marie_short_course/Singapore course 2022/20220602/exercises/data")

##-R libraries to install:
#devtools::install_github("Fgraziano/design2phase")
#install.packages(c("haven","knitr","survey"))
##-Load the R libraries
library(design2phase)
library(haven)
library(knitr)
library(survey)


################################################################################
##                               Question 1      
##
##-1. For this exercise we will refer to the CASS data that was analysed in 
##    Reilly AJE 1996. We are interested in estimating the association between 
##    left ventricular end diastolic blood pressure (LVEDP) and operative 
##    mortality.  
##
##    Let's suppose that the binary outcome (operative mortality) and a number 
##    of demographic and clinical covariates (age, sex, weight, surgery, 
##    angina, chf) are available for the full cohort (N=8088 patients), but 
##    that we have to retrieve data on LVEDP by clinical charts. We expect a 
##    High LVEDP in 23% (LVEDP>17 mmHg) of the subjects and an effect size (OR) 
##    nearly 1.65.  
##
##    From the CASS cohort we know that:
##    - the proportion of females is 16%
##    - the hazard ratio on operative mortality for females compared to males 
##      is approximately 2.19
################################################################################


################################################################################
##                               Question 1 (i)     
##
##-(i) Due to a budget limitation we need to sample a subgroup of subjects for 
##     measuring LVEDP (n=400 subjects). Use the PowerIIPhase() function to 
##     find the power for different designs: a simple random sample (SRS), 
##     a random case-control (CC) sample and a CC sample stratified on sex. 
##     Compare the results. Is there a gain in power by stratifying for sex?   
##  
##     Be aware that computation time might be long. For this reason, we suggest 
##     to limit number of simulations to 150 (B=150). Please note that in order 
##     to simulate CASS data we need to specify the following parameters 
##     (tau = 1, lambda = 0.02; cens = 0, see definition below at Hints). 
################################################################################


ex1<-PowerIIphase(pBM =0.23, betaBM = 0.50,pstrata = 0.16,
                  betastrata = 0.78, cens=0,tau=1, N=8088,lambda=0.02, n=400, 
                  seed=123, B=150)
ex1


################################################################################
##                               Question 1 (ii)
##
##-(ii) Estimate the power of each design mentioned in (i) varying the 
##      subgroup sample size from 300 to 400patients. Show the power curves 
##      using the function PlotPower(). 
################################################################################

ex2<-PowerIIphase(pBM =0.23, betaBM = 0.50,pstrata = 0.16,
                  betastrata =  0.78, cens=0,tau=1, N=8088,lambda=0.02, 
                  n=c(300, 400), seed=123, B=150)
ex2
PlotPower(ex2)


################################################################################
##                               Question 1 (iii)
##
##-(iii) Suppose that we have an auxiliary variable of LVEDP in the full cohort 
##       (note: auxiliary variable is a variable associated with the marker of 
##       interest) with sensitivity and specificity of 0.80 and 0.80 
##       respectively (where sensitivity is defined as Prob(CF=high|LVEDP=high) 
##       and specificity is Prob(CF=low|LVEDP=low)). Compute the power for a 
##       case-control design stratified for an auxiliary variable and compare 
##       it with the one obtained in (i).
################################################################################

ex3<-PowerIIphase(pBM =0.23, betaBM = 0.50,pstrata = 0.16,
                  betastrata =  0.78,acc.aux=c(0.80, 0.80), cens=0,tau=1, 
                  N=8088,lambda=0.02, n=400, seed=123, B=150)
ex3


################################################################################
##                               Question 1 (iv)
##-(iv) (optional) The full cohort data is in chap8_CASS.dta. Perform the 
##      following steps:
##      (a) Create a categorical variable for LVEDP considering high level 
##          with LVEDP(lve) > 17 mmHg. 
##      (b) Randomly select a case-control subsample of 400 subjects.
##      (c) Run weighted logistic regression analysis using survey package to 
##          obtain the crude OR for dichotomized LVEDP from the case-control 
##          subsample in step (b).
##      (d) (advanced) Reproduce step (b) and (c) 100 times and count the number
##          of subsamples that achieved a significant effect of LVEDP 
##          (type I error of 0.05). 
##
##      NOTE: Your sample size may be less than 400 in the final analysis 
##            because some cohort members are missing LVEDP!
################################################################################

#1- read in data
CASSdata <- haven::read_dta("chap8_CASS.dta")
CASSdata <- CASSdata[complete.cases(CASSdata$lve) , ]

#2- step (a)
head(CASSdata)
CASSdata$lveB<-ifelse(CASSdata$lve>17,1,0)
table(CASSdata$lveB) 
CASSdata$id<-seq(1:dim(CASSdata) [1])

#3- step (b)
#CASE - CONTROL Select all cases
set.seed(123)
cases<-CASSdata$id[CASSdata$mort==1]
cntl<-sample(CASSdata$id[!CASSdata$mort==1],200, replace = FALSE)
idsampled<-c(cases,cntl);
campione<-CASSdata[CASSdata$id %in% idsampled,]
CASSdata$incl<-ifelse(CASSdata$id %in% idsampled,1,0);
DATse<-CASSdata[CASSdata$incl==1,]
CASSdata$mort<-as.factor(CASSdata$mort)

table(CASSdata$lveB)

#4- step (c)
two_stage_design <- survey::twophase(id = list(~id, ~id), 
                                     subset = ~incl, 
                                     strata = list(NULL, ~mort), 
                                     data = CASSdata)

survey_model <- survey::svyglm(mort ~ lveB, family = binomial("logit"), 
                               design = two_stage_design)

tmp1 <- data.frame(summary(survey_model)$coef)
tmp1$OR <- round(exp(tmp1[,1]),3)
tmp1$SE <- round(tmp1[,2],3)
tmp1$lower.CI <- round(exp(tmp1[,1] - 1.96* tmp1[,2]),3)
tmp1$upper.CI <- round(exp(tmp1[,1] + 1.96* tmp1[,2]),3)
names(tmp1)[4] <- 'p-value'
knitr::kable(tmp1[,c('OR','SE', 'lower.CI','upper.CI','p-value')])


#5- step (d)
B<-150

sample.pval<-NULL
for (i in 1:B) {
  
  #5a: step (b)
  cases<-CASSdata$id[CASSdata$mort==1]
  cntl<-sample(CASSdata$id[!CASSdata$mort==1],200, replace = FALSE)
  idsampled<-c(cases,cntl);
  campione<-CASSdata[CASSdata$id %in% idsampled,]
  CASSdata$incl<-ifelse(CASSdata$id %in% idsampled,1,0)
  DATse<-CASSdata[CASSdata$incl==1,]
  CASSdata$mort<-as.factor(CASSdata$mort)

  #5b: step (c)
  two_stage_design <- survey::twophase(id = list(~id, ~id), 
                                       subset = ~incl, 
                                       strata = list(NULL, ~mort), 
                                       data = CASSdata)
  survey_model <- survey::svyglm(mort ~ lveB, family = binomial("logit"), 
                                 design = two_stage_design)
  sample.pval[i]<- summary(survey_model)$coefficients[2,4]
}

sample.pval
signp<-ifelse(sample.pval<0.05,1,0)
P<-sum(signp)/B
P
