//----------------------------------------------------------------------------//
//=Purpose: Exercise 4.2 of Beyond Classic Designs & Analysis for Health      //
//          Data in Singapore (27 June to 8 July 2022)                        //
//
//=Data required: 
//  - Framingham_HDL.dta, 
//  - Hpylori_extract.dta
//
//=Stata packages/libraries required:
//  - sxd2: Computing optimal sampling designs for two-stage studies
//    *optfixn: to select an optimal second-stage sample of a fixed size
//    *optprec: to select the optimal first and second stage samples that 
//              minmise cost for a required precision for one coefficient
//    *optbud: to select the optimal first and second stage samples for a 
//             specifed budget that minimize the variance of a coefficient.
//
//=Created by:
//  - Zihan 
//  - Francesca Graziano 
//  - Zilu Feng
//  - Zihan 
//  - Chuen Seng Tan
//  - Marie Reilly
//
//=Changes (if any):
//  -DDMMYYYY: Describe
//
//----------------------------------------------------------------------------//

//-Set the working directory to the data folder where the datasets are saved:
cd "D:/nbox/Education/Marie_short_course/Singapore course 2022/20220602/exercises/data"

////////////////////////////////////////////////////////////////////////////////
//                               Question 1      
//
//1- The data set wtpilot.dta contains the binary outcome (operative mortality: mort) //   and a number of demographic and clinical covariates (age: age, sex: sex, weight: //   weight, surgery: surg, angina: angina, chf: chf, lvedp: lvedp) for a pilot sample //   of size 118 selected from the full CASS data that was analysed in Reilly AJE 
//   1996. The original plan for the pilot data was to sample 10 observations from 
//   each of the twelve strata defined by the outcome, sex, and a 3-category weight 
//   variable, but as seen in the following table from the paper, one category had 
//   only 8 observations:
//
//   [see table in exercise]
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//                               Question 1 (i)     
//
//-i. Run the optfixn command with the wtpilot data and the population numbers in the 
//    12 strata from Table 5 above, to obtain an optimal second-stage sample of 1000 
//    which minimizes the variance of left ventricular end diastolic blood pressure 
//    (LVEDP) in the following logistic model:
//
//    mort = α + sex + weight + age + angina + chf + lvedp + surg
//
//    (You will note some differences from Table 6 in the AJE paper, which used a 
//    different pilot sample).
////////////////////////////////////////////////////////////////////////////////

//load dataset (pilot):
use wtpilot.dta, clear

//take a look at the 12 strata:
coding mort sex wtcat

//enter the vector of first-stage sample sizes from Table 5 as follows:
matrix fstsamp=(160, 1083, 5418, 440, 407, 378, 8, 33, 103, 18, 26, 14)'

//obtain the optimal second-stage sample sizes (fixed number of 1st and 2nd stage sample):
optfixn mort sex wt age angina chf lve surg, first(sex wtcat) n1(fstsamp) n2(1000) var(7) 

//Therefore, to minimize the variance of left ventricular end diastolic blood pressure, we need to sample all the available cases, 15% of controls in stratum 1, 28.8% of controls in stratum 2, 5.3% of controls in stratum 3, 18.4% of controls in stratum 4, 10% of controls in stratum 5, 14% of controls in stratum 6.


////////////////////////////////////////////////////////////////////////////////
//                               Question 1 (ii)     
//
//-ii. The full cohort data from which the pilot sample was drawn is in the dataset 
//     chap8_CASS.dta. Selected another balanced pilot sample of the same size as in 
//     (i), and compare the optimal design obtained using this pilot data to the 
//     design in (i). (NOTE: Your sample size may be less than 1000 in the analysis 
//     because some cohort members are missing LVEDP!)
////////////////////////////////////////////////////////////////////////////////

//load the full cohort data:
use chap8_CASS, clear

//check the first-stage sample size in each stratum:
coding mort sex wtcat

//create strata acoording to mort, sex and wtcat:
egen strat = group(mort sex wtcat)
codebook strat
	
//random sample 10 ppl in each stratum:
set seed 1810
gen u = runiform()
sort strat u
gen select = 0
by strat: replace select = 1 if _n <= 10
tabulate strat select

//save the new sample:
keep if select==1
save pilot1, replace

//enter the vector of first-stage sample sizes from Table 5 as follows:
matrix fstsamp=(160, 1083, 5418, 440, 407, 378, 8, 33, 103, 18, 26, 14)'

//obtain the optimal second-stage sample sizes (fixed number of 1st and 2nd stage sample):
optfixn mort sex wt age angina chf lve surg, first(sex wtcat) n1(fstsamp) n2(1000) var(7) 

//To minimize the variance of left ventricular end diastolic blood pressure we need to sample all the available cases, 18.4% of controls in stratum 1, 5.7% of controls in stratum 2, 8.7% of controls in stratum 3, 32.7% of controls in stratum 4, 12.7% of controls in stratum 5, 10.5% of controls in stratum 6


////////////////////////////////////////////////////////////////////////////////
//                               Question 1 (iii)     
//
//-iii. If the study in (i) or (ii) cost an average of €1 for each first stage and 
//      each second stage observation, then the total cost would be approximately 
//      €10,000. Use the optbud command to find the optimal design for this budget if 
//      a first-stage observation costs €5 and a second stage observation costs €50. 
//      What is the total study size, the total and stratum-specific second-stage 
//      sample size, the total cost of this study and the variance achieved for the 
//      coefficient of LVEDP.
////////////////////////////////////////////////////////////////////////////////

//load the dataset:
use wtpilot.dta, clear

//enter the vector of prevalence calculated above as follows:
matrix prev=(160/8088,1083/8088,5418/8088,440/8088,407/8088,378/8088,8/8088,33/8088,103/8088,18/8088,26/8088,14/8088)'

//obtain the optimal second-stage sample sizes (fixed budget & cost of sample at 1st and 2nd stage):
optbud mort sex wt age angina chf lve surg, first(sex wtcat) prev(prev) var(7) b(10000) c1(5) c2(50)

//display total second stage sample size:
di 2+28+26+7+4+5+1+4+13+2+3+2

//Therefore, the total study size is 1023; total second stage sample size is 97; total cost is 9965; variance achieved for the coefficient of LVEDP is 0.00109187.	
	

////////////////////////////////////////////////////////////////////////////////
//                               Question 1 (iv)     
//
//-iv. Use the optprec command to design a study that would achieve the variance you 
//     found in (iii), for minimum cost, assuming the same costs as before for a first 
//     and second-stage observation (€5 and €50 respectively). What do you notice?
////////////////////////////////////////////////////////////////////////////////

optprec mort sex wt age angina chf lve surg, first(sex wtcat) prev(prev) var(7) prec(.00109187) c1(5) c2(50) 

//The optimal design for this case is almost the same as its counterpart in (iii), as these are simply two ways of asking the same question.


////////////////////////////////////////////////////////////////////////////////
//                               Question 1 (v)     
//
//-v. Show that for the same budget as in (iv), if C1 = €5 and C2 = €10 the optimal 
//    design is a study approximately 50% larger than in (iv), which would sample at 
//    least twice as many controls in each stratum.
////////////////////////////////////////////////////////////////////////////////

optbud mort sex wt age angina chf lve surg, first(sex wtcat) prev(prev) var(7) b(9960) c1(5) c2(10)


////////////////////////////////////////////////////////////////////////////////
//                               Question 2     
//
//-2. The data set ectopic.dta consist of a total of 979 observations from a 
//    case-control study of ectopic pregnancy, with the following variables:
//    *y       a binary indicator for case (i.e. ectopic pregnancy) or control 
//    *gonn	   history of gonorrhea infection 
//    *contr   use of contraception
//    *sexptn  multiple sex partners 
//    *chlam   positive for chlamydia antibody (available for 327 individuals)
//    *z	   a stratum variable constructed from gonn, contr and sexptn.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//                               Question 2 (i)     
//
//-i. Assusing the 327 individuals with chlamydia results were sampled from the strata 
//    defined by gonn, contr and sexptn (i.e. the variable “z”), analyse all the data 
//    using weighted logistic regression or the meanscor command,  to estimate the 
//    effect of exposure to chlamydia antibody on the risk of ectopic pregnancy, 
//    adjusted for the other variables.
////////////////////////////////////////////////////////////////////////////////

//load the dataset:
use ectopic.dta, clear

//estimate the effect of exposure to chlamydia antibody on the risk of ectopic pregnancy, adjusted for the other variables (note: if you want to get coefficients instead of ORs, then should add an option "odd(0)"):

meanscor y gonn contr sexptn chlam, first(gonn contr sexptn) second(chlam)

//Alternatively (note: if we use this syntax, need to be aware that the order of the variables in the "Variables" window is alpahbetic, so not necessarily the order in the data set. In the data set provided, the order is: y gonn contr sexptn chlam z, where z is a stratum variable created from combinations of first-stage variables: gonn, contr and sexptn):
meanscor y gonn-chlam, first(z) second(chlam)


////////////////////////////////////////////////////////////////////////////////
//                               Question 2 (ii)     
//
//-ii. Verify the numbers and % in the final column of the following table (Table 8.2 
//     in CES 8.2) using suitable cross-tabulation commands (or the coding command in 
//     Stata).
////////////////////////////////////////////////////////////////////////////////

//strate in full data:
coding y contr sexptn

//strata in second-stage:
drop if chlam==.
coding y contr sexptn

//calculate percentage:
di 13/56 41/138 36/146 102/375 11/26 97/189 5/9 22/40

*Alternatively:*
use ectopic.dta, clear
	
//create strata acoording to y, contr and sexptn:
egen strat = group(y contr sexptn)

//count number of ppl in each creasted stratum:
bysort strat: gen stratsize = _N

//in each strata, count number of ppl whose chlam data is available:
bysort strat: egen stratselected = total(chlam != .)

//calculate % and show output:
gen percentage = stratselected / stratsize
keep stratsize stratselected percentage
duplicates drop
list 


////////////////////////////////////////////////////////////////////////////////
//                               Question 2 (iii)     
//-iii. Verify that the adjusted OR for chlamydia is approximately the same if we 
//      assume the sampling depended only on contraceptive use and sex partners.
////////////////////////////////////////////////////////////////////////////////

use ectopic.dta, clear
meanscor y gonn-chlam, first(contr sexpt) second(chlam)

//there is no big difference compared to results in (i), which is sampled by gonn, contr, sexptn


////////////////////////////////////////////////////////////////////////////////
//      The remaining sections of this exercise will use these two (contr, sexptn) as 
//      first stage variables as in CES 8.2.
//
//                               Question 2 (iv)     
//
//-iv. Using the available 327 complete observations as pilot data, verify that for a 
//     sample of this size (i.e. optfixn with n=327), the minimum achievable variance 
//     for the coefficient of chlamydia is .0674. How does this compare to the actual 
//     variance achieved by the investigators with the data collected in the study?
////////////////////////////////////////////////////////////////////////////////

use ectopic.dta, clear

//strata in full data:
coding y contr sexpt

//use just second-stage data and display strata in second-stage data (as pilot):
keep if chlam !=.
coding y contr sexpt

//enter the vector of first-stage sample sizes as follows:
matrix fstsamp=(56, 138, 146, 375, 26, 189, 9, 40)'

//obtain the optimal second-stage sample sizes (fixed number of 1st and 2nd stage sample):
optfixn y contr sexpt chlam gonn, first(contr sexpt) n1(fstsamp) n2(327) var(4) 

//The precision is improved when comparing to (i).


////////////////////////////////////////////////////////////////////////////////
//                               Question 2 (iv)     
//
//-v. Use the optprec command to verify that if the first and second stage 
//    observations cost €5 and €50 respectively, the design that achieves the same 
//    variance as in (iv) for minimum cost is:
//
//    First stage sample size:  672
//    Sampling fractions in strata Z=1,2,…8  =  (.56, 1, .08,  .22, .97, .77, 1, 1)
////////////////////////////////////////////////////////////////////////////////

use ectopic.dta, clear
	
//strata in full data:
coding y contr sexpt
keep if chlam !=.
	
//enter the vector of prevalence as follows:
matrix prev=(56/979, 138/979, 146/979, 375/979, 26/979, 189/979, 9/979, 40/979)'
	
//run *optprec* command:
optprec y gonn contr sexptn chlam, first(contr sexptn) prev(prev) var(5) prec(.067) c1(5) c2(50) 
